package com.zh.project0512.controller.manage;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.AdminCheckAuthorityAnnotation;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.model.AppMessage;
import com.zh.project0512.model.UserEffectRecords;
import com.zh.project0512.model.Video;
import com.zh.project0512.model.VideoEffectSetting;
import com.zh.project0512.service.IUserEffectRecordsService;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.service.IVideoEffectSettingService;
import com.zh.project0512.service.IVideoService;
import com.zh.project0512.utils.AppMessageUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-18
 */
@RestController
@RequestMapping("/manage/video")
@Tag(name = "视频")
public class VideoController {
    @Autowired
    private IVideoService videoService;
    @Autowired
    private IUserService userService;
    @Autowired
    private IVideoEffectSettingService videoEffectSettingService;
    @Autowired
    private IUserEffectRecordsService userEffectRecordsService;

    @Data
    static class VParam {
        private Integer pageNum;
        private Integer pageSize;
        @Schema(title = "视频状态", description = "1待审核；2审核通过；3审核拒绝")
        @NotNull(message = "视频状态不能为空")
        @Min(value = 1, message = "视频状态范围：1待审核；2审核通过；3审核拒绝")
        @Max(value = 3, message = "视频状态范围：1待审核；2审核通过；3审核拒绝")
        private Integer status;
        @Schema(title = "申请人名称")
        private String providerName;
        @Schema(title = "部门id")
        private Integer departmentId;
        @Schema(title = "任务标题")
        private String taskTitle;
    }

    @Operation(summary = "审核视频列表")
    @PostMapping("/list")
    @AdminTokenValid
    public Result list(@Validated @RequestBody VParam param) {
        QueryWrapper<Video> qw = new QueryWrapper<>();
        qw.eq("t1.status", param.getStatus());
        qw.like(param.providerName != null, "t2.name", param.getProviderName());
        qw.eq(param.getDepartmentId() != null, "t3.departmentId", param.getDepartmentId());
        qw.like(param.getTaskTitle() != null, "t4.title", param.getTaskTitle());
        return Result.success(videoService.pageList(MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize()), qw));
    }

    @Data
    static class VVParam {
        @NotNull(message = "id不能为空")
        @Min(value = 1, message = "id最小值为1")
        @Schema(title = "视频id")
        private Integer id;
        @NotNull(message = "审核状态不能为空")
        @Min(value = 2, message = "审核状态为：2通过；3拒绝")
        @Max(value = 3, message = "审核状态为：2通过；3拒绝")
        @Schema(title = "审核状态", description = "2通过；3拒绝")
        private Integer status;
        @Schema(title = "播放次数", description = "抖音1快手2字段")
        private Integer playNum;
        @Schema(title = "点赞次数")
        private Integer commendNum;
        @Schema(title = "收藏次数", description = "抖音1快手2字段")
        private Integer collectionNum;
        @Schema(title = "评论次数")
        private Integer commentNum;
        @Schema(title = "转发次数", description = "视频号4字段")
        private Integer reSendNum;
        @Schema(title = "推荐次数", description = "视频号4字段")
        private Integer recommendNum;
        @Schema(title = "拒绝理由")
        private String reason;
    }

    @Operation(summary = "传播值计算")
    @PostMapping("/cal")
    @AdminTokenValid
    public Result cal(@Validated @RequestBody VVParam param) {
        JSONObject jsonObject = new JSONObject();
        List<VideoEffectSetting> list = videoEffectSettingService.list();
        Integer effectResult = cal(param.getPlayNum(), list.get(0).getEffectTimes()) + cal(param.getCommendNum(), list.get(1).getEffectTimes())
            + cal(param.getCollectionNum(), list.get(2).getEffectTimes()) + cal(param.getCommentNum(), list.get(3).getEffectTimes())
            + cal(param.getReSendNum(), list.get(4).getEffectTimes()) + cal(param.getRecommendNum(), list.get(5).getEffectTimes());
        jsonObject.put("effectResult", effectResult);

        return Result.success(jsonObject, "计算完成");
    }

    @Operation(summary = "视频审核")
    @PostMapping("/valid")
    @Transactional
    @AdminTokenValid
    @AdminCheckAuthorityAnnotation(jurisdictionId = "13")
    public Result valid(@Validated @RequestBody VVParam param) {
        int id = param.getId();
        Video video = videoService.getById(id);
        Integer originEffectResult = video.getEffectResult() == null ? 0 : video.getEffectResult();
        if (param.getStatus() == 3) {
            String reason = param.getReason();
            if (reason == null) {
                return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "请说明拒绝理由");
            }
            UpdateWrapper<Video> set = new UpdateWrapper<>();
            set.eq("id", id).set("status", 3).set("reason", param.getReason()).set("updateAt", LocalDateTime.now());
            boolean op = videoService.update(set);
            if (!op) {
                return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "无效操作");
            }
            AppMessageUtil.sendMessage(new AppMessage().setTitle("传播效果审核结果").setDescription("您的抖音传播效果审核未通过，未通过原因：" + param.getReason() + "。")
                .setUrl("pages/quest/subpage/detail/detail?id=" + video.getTaskId()).setReceiverId(video.getUserId()));
            return Result.success("", "审核拒绝完成");
        }
        UpdateWrapper<Video> up = new UpdateWrapper<>();
        up.eq("id", id).ne("status", 0);
        up.set(param.getPlayNum() != null, "playNum", param.getPlayNum());
        up.set(param.getCommendNum() != null, "commendNum", param.getCommendNum());
        up.set(param.getCollectionNum() != null, "collectionNum", param.getCollectionNum());
        up.set(param.getCommentNum() != null, "commentNum", param.getCommentNum());
        up.set(param.getReSendNum() != null, "reSendNum", param.getReSendNum());
        up.set(param.getRecommendNum() != null, "recommendNum", param.getRecommendNum());
        up.set("status", 2);
        List<VideoEffectSetting> list = videoEffectSettingService.list();
        Integer effectResult = cal(param.getPlayNum(), list.get(0).getEffectTimes()) + cal(param.getCommendNum(), list.get(1).getEffectTimes())
            + cal(param.getCollectionNum(), list.get(2).getEffectTimes()) + cal(param.getCommentNum(), list.get(3).getEffectTimes())
            + cal(param.getReSendNum(), list.get(4).getEffectTimes()) + cal(param.getRecommendNum(), list.get(5).getEffectTimes());
        up.set("effectResult", effectResult);
        up.set("updateAt", LocalDateTime.now());
        videoService.update(up);
        userEffectRecordsService.save(new UserEffectRecords().setUserId(video.getUserId()).setEffectResultTran(effectResult-originEffectResult).setCreateAt(LocalDateTime.now()));
        String platForm = "未知平台";
        switch (video.getType()) {
            case 1:
                platForm = "抖音";
                break;
            case 2:
                platForm = "快手";
                break;
            case 3:
                platForm = "朋友圈";
                break;
            case 4:
                platForm = "视频号";
                break;
            default:
        }
        AppMessageUtil.sendMessage(new AppMessage().setTitle("传播效果审核结果").setDescription("恭喜您，" + platForm + "传播效果审核通过，平台评定传播值为" + effectResult + "。")
            .setUrl("pages/quest/subpage/detail/detail?id=" + video.getTaskId()).setReceiverId(video.getUserId()));
        return Result.success("", "审核完成");
    }

    static int cal(Integer value, Integer effectTimes) {
        if (value == null || value == 0 || effectTimes == null || effectTimes == 0) {
            return 0;
        }
        return value * effectTimes;
    }

}

